@@ -6,40 +6,28 @@ To connect to SeaVoice STT & TTS server to finish speech recognizing and synthes
 """
 
 
-from typing import (
-    Optional,
-    Dict,
-    Callable,
-    Text
-)
-import websockets
-from ws4py.client.threadedclient import WebSocketClient
-import websocket
-import urllib
-import asyncio
-from seavoice_sdk import audio
-import threading
-import json
-import warnings
-from seavoice_sdk import logger
-
-import base64
-import hashlib
-import hmac
-import os
-import ssl
-import time
 import _thread as thread
+import asyncio
+import base64
 import enum
 import io
-from seavoice_sdk.utils import Wave
-import requests
+import json
+import ssl
+import threading
+import urllib
+import uuid
+import warnings
+from typing import Callable, Dict, Optional, Text
 
 # used by SlothWebsocketParam
-from wsgiref.handlers import format_date_time
-from urllib.parse import urlencode
-from datetime import datetime
-from time import mktime
+import requests
+import websocket
+import websockets
+import ws4py.messaging
+from ws4py.client.threadedclient import WebSocketClient
+
+from seavoice_sdk import audio, logger
+from seavoice_sdk.utils import Wave
 
 svc_logger = logger.get_SVCLogger(__name__)
 
@@ -49,41 +37,29 @@ API_SERVER_STT_SERVICE_URL = "https://suite.seasalt.ai/api/v1/speech/stt_server_
 API_SERVER_STT_SERVICE_URL_TOKEN = "https://suite.seasalt.ai/api/v1/speech/speech_to_text"
 
 SUPPORTED_AUDIO_FORMAT = {
-    'riff-8khz-16bit-mono-pcm': {
-        "sample_rate": 8000,
-        "sample_width": 2,
-        "channels": 1
-    },
-    'riff-16khz-16bit-mono-pcm': {
-        "sample_rate": 16000,
-        "sample_width": 2,
-        "channels": 1
-    },
-    'riff-22khz-16bit-mono-pcm': {
-        "sample_rate": 22050,
-        "sample_width": 2,
-        "channels": 1
-    }
+    "riff-8khz-16bit-mono-pcm": {"sample_rate": 8000, "sample_width": 2, "channels": 1},
+    "riff-16khz-16bit-mono-pcm": {"sample_rate": 16000, "sample_width": 2, "channels": 1},
+    "riff-22khz-16bit-mono-pcm": {"sample_rate": 22050, "sample_width": 2, "channels": 1},
 }
 ERROR_IN_SEND_THREAD = 11000
 
 
-class SpeechConfig():
-    """ Setup speech arguments """
+class SpeechConfig:
+    """Setup speech arguments"""
 
     def __init__(
         self,
-        host: Optional[Text] = '',
-        account_id: Text = '',
-        password: Text = '',
-        apikey: Text = '',
-        speech_recognition_language: Text = 'zh-TW',
+        host: Optional[Text] = "",
+        account_id: Text = "",
+        password: Text = "",
+        apikey: Text = "",
+        speech_recognition_language: Text = "zh-TW",
         speech_recognition_punctuation: bool = False,
-        speech_synthesis_language: Text = 'zh-TW',
-        speech_synthesis_voice_name: Text = 'Lin_Xiaomei',
-        speech_synthesis_output_format_id: Text = 'riff-16khz-16bit-mono-pcm',
+        speech_synthesis_language: Text = "zh-TW",
+        speech_synthesis_voice_name: Text = "Lin_Xiaomei",
+        speech_synthesis_output_format_id: Text = "riff-16khz-16bit-mono-pcm",
         speech_synthesis_output_pitch: int = 0.0,
-        speech_synthesis_output_speed: int = 1.0
+        speech_synthesis_output_speed: int = 1.0,
     ):
         """Init speech arguments"""
         self.__host = host
@@ -99,7 +75,7 @@ class SpeechConfig():
         except KeyError:
             raise ValueError("Unavailable format id")
         self.__speech_synthesis_output_format_id = speech_synthesis_output_format_id
-        self.__speech_synthesis_format_label = 'X-SeasaltAI-OutputFormat'
+        self.__speech_synthesis_format_label = "X-SeasaltAI-OutputFormat"
         self.__speech_synthesis_output_pitch = speech_synthesis_output_pitch
         self.__speech_synthesis_output_speed = speech_synthesis_output_speed
 
@@ -214,10 +190,12 @@ class SpeechConfig():
 
     def get_login_token(self) -> bool:
         """login on api server, get login token"""
-        headers = {'Content-Type': 'text/plain'}
-        response = requests.post(API_SERVER_LOGIN_URL, headers=headers, json={'account_id': self.__account_id, 'password': self.__password})
+        headers = {"Content-Type": "text/plain"}
+        response = requests.post(
+            API_SERVER_LOGIN_URL, headers=headers, json={"account_id": self.__account_id, "password": self.__password}
+        )
         if response.status_code == 200:
-            self.login_token = response.json()['token']
+            self.login_token = response.json()["token"]
             return True
         else:
             svc_logger.error(f"Login error: {response.status_code} for account_id: {self.__account_id}")
@@ -227,15 +205,69 @@ class SpeechConfig():
         """Enable audio loggin"""
         pass
 
-
-class Recognizer(WebSocketClient):
+class Recognizer():
     """Base class of recognizer"""
+    
+    class RecognizerWebSocket(WebSocketClient):
+        """Helper class to handle websocket communication"""
+        def __init__(self, url, headers, recognizer):
+            # init websocket client
+            self.recognizer = recognizer
+            super().__init__(url=url, headers=headers)
 
-    def __init__(
-        self,
-        speech_config: SpeechConfig,
-        audio_config: Optional[audio.AudioConfig] = None
-    ):
+        def received_message(self, message: ws4py.messaging.TextMessage) -> None:
+            """Receive result from websocket"""
+            svc_logger.info(f"RECEIVED MESSSGE {message}")
+            try:
+                evt = json.loads(str(message))
+                if "id" in evt:
+                    self.recognizer.session_id = evt["id"]
+                if evt["status"] == 0:
+                    if "result" in evt:
+                        if evt["result"]["final"]:
+                            if self.recognizer._Recognizer__recognized_event.is_set:
+                                evt_res = EventResults(evt)
+                                self.recognizer._Recognizer__recognized_event.callback(evt_res)
+                                if self.recognizer._Recognizer__recognize_once is True:
+                                    self.recognizer._Recognizer__recognize_once = False
+                                    self.close()
+                        else:
+                            if self.recognizer._Recognizer__recognizing_event.is_set:
+                                evt_res = EventResults(evt)
+                                self.recognizer._Recognizer__recognizing_event.callback(evt_res)
+                else:
+                    if "message" in evt:
+                        evt["message"] = "Received error from server: " + f"{evt['message']}"
+                    else:
+                        evt["message"] = "Got error from server."
+                    raise RuntimeError(evt)
+
+            except Exception as e:
+                svc_logger.error(f"Exception in receive_message: {str(message)}, e: {e}")
+                if self.recognizer._Recognizer__canceled_event.is_set:
+                    evt_res = EventResults(evt)
+                    self.recognizer._Recognizer__canceled_event.callback(evt_res)
+                self.recognizer.stop_continuous_recognition_async()
+
+        def closed(self, code, reason):
+            """Close websocket"""
+            svc_logger.info(f"Closed down: {code}, {reason}")
+            if self.recognizer._Recognizer__session_stopped_event.is_set:
+                evt = {"session_id": self.recognizer.session_id, "status": code, "messge": reason}
+                evt_res = EventResults(evt)
+                self.recognizer._Recognizer__session_stopped_event.callback(evt_res)
+            self.close()
+            self = None
+
+        def opened(self):
+            """Open websocket"""
+            svc_logger.info("websocket opened")
+            if self.recognizer._Recognizer__session_started_event.is_set:
+                evt = {"session_id": self.recognizer.session_id, "status": 0, "message": "recognition started"}
+                evt_res = EventResults(evt)
+                self.recognizer._Recognizer__session_started_event.callback(evt_res)
+
+    def __init__(self, speech_config: SpeechConfig, audio_config: Optional[audio.AudioConfig] = None, monitor_session: bool = False, itn: bool = True, punctuation: bool = True):
         """Initialize speech recognizer"""
         # load speech_config
         self.__speech_config = speech_config
@@ -247,18 +279,22 @@ class Recognizer(WebSocketClient):
 
         if not speech_config.apikey:
             if speech_config.get_login_token() is False:
-                svc_logger.error(f"Failed to login API server for {self.__account_id}, please make sure account_id and password are right.")
+                svc_logger.error(
+                    f"Failed to login API server for {self.__account_id}, please make sure account_id and password are right."
+                )
                 return None
             self.__login_token = speech_config.login_token
 
             # get an available stt server url and speech token
             if self.get_server_url_token() is False:
-                svc_logger.error(f"Can't get an available stt server and speech token, please confirm language setting, then try again.")
+                svc_logger.error(
+                    "Can't get an available stt server and speech token, please confirm language setting, then try again."
+                )
                 return None
         else:
             self.__token = speech_config.apikey
-            if self.get_server_url() is False:
-                svc_logger.error(f"Can't get an available stt server, please confirm language setting, then try again.")
+            if not self.get_server_url():
+                svc_logger.error("Can't get an available stt server, please confirm language setting, then try again.")
                 return None
         self.__ws_uri = f"{self.__host}?token={self.__token}&punctuation={self.__speech_recognition_punctuation}"
 
@@ -272,9 +308,7 @@ class Recognizer(WebSocketClient):
             if audio_config.stream is not None:
                 self.__input_stream = audio_config.stream
                 self.__content_type = urllib.parse.urlencode(
-                    [
-                        ("content-type", self.__input_stream.stream_format.content_type)
-                    ]
+                    [("content-type", self.__input_stream.stream_format.content_type)]
                 )
             elif audio_config.filename is not None:
                 self.__input_filename = audio_config.filename
@@ -290,31 +324,50 @@ class Recognizer(WebSocketClient):
         self.__canceled_event = EventSignal()
         self.session_id = None
 
+        # stats for monitoring session
+        self.reconnecting = False
+        self.monitor_session = monitor_session
+        self.connection_id = str(uuid.uuid4())
+        self.seconds_received = 0
+
+        # Speech Recognition itn options
+        self.itn = itn
+        self.punctuation = punctuation
+        
+        if self.__input_stream:
+            sample_rate = self.__input_stream.stream_format.samples_per_second
+        else:
+            sample_rate = 48000
+        svc_logger.info(f"Sample Rate: {sample_rate}, ITN: {self.itn}, Punctuation: {self.punctuation}")
+
         # init websocket client
-        super().__init__(url=self.__ws_uri)
+        self.ws = self.RecognizerWebSocket(self.__ws_uri, [("sample_rate",sample_rate), ("itn",self.itn), ("punctuation",self.punctuation)], self)
 
     def get_server_url_token(self) -> bool:
         """get stt server url and speech token by login token"""
-        headers = {'Content-Type': 'text/plain', 'token': self.__login_token}
-        response = requests.post(API_SERVER_STT_SERVICE_URL_TOKEN, headers=headers,
-                                 json={'language': self.__speech_recognition_language})
+        headers = {"Content-Type": "text/plain", "token": self.__login_token}
+        response = requests.post(
+            API_SERVER_STT_SERVICE_URL_TOKEN, headers=headers, json={"language": self.__speech_recognition_language}
+        )
         if response.status_code == 200:
             response_json = response.json()
-            self.__host = response_json['server_url']
-            self.__token = response_json['token']
+            self.__host = response_json["server_url"]
+            self.__token = response_json["token"]
             return True
         else:
-            svc_logger.error(f"Error while getting stt server url and token: {response.status_code} for account_id: {self.__account_id}")
+            svc_logger.error(
+                f"Error while getting stt server url and token: {response.status_code} for account_id: {self.__account_id}"
+            )
             return False
 
     def get_server_url(self) -> bool:
         """get stt server url by apikey"""
-        headers = {'Content-Type': 'text/plain', 'speech_token': self.__token}
+        headers = {"Content-Type": "text/plain", "speech_token": self.__token}
         url = f"{API_SERVER_STT_SERVICE_URL}?language={self.__speech_recognition_language}"
         response = requests.get(url, headers=headers)
         if response.status_code == 200:
             response_json = response.json()
-            self.__host = response_json['server_url']
+            self.__host = response_json["server_url"]
             return True
         else:
             svc_logger.error(f"Error while getting stt server url: {response.status_code} for account_id: {self.__account_id}")
@@ -322,7 +375,7 @@ class Recognizer(WebSocketClient):
 
     def recognize_once(self):
         """recognize once"""
-        self.connect()
+        self.ws.connect()
         if self.__input_filename is None:
             raise RuntimeError("No filename provided!")
         self.__recognize_once = True
@@ -332,18 +385,23 @@ class Recognizer(WebSocketClient):
                 frame_data = wav.read(frame)
                 if len(frame_data) == 0:
                     break
-                self.send(frame_data, binary=True)
-            self.send("EOS")
+                self.ws.send(frame_data, binary=True)
+            self.ws.send("EOS")
 
     def start_continuous_recognition_async(self):
         """Start continuous speech recognition in asynchronized way"""
         self.is_running = True
-        self.connect()
-        recognition_thread = threading.Thread(
-            target=self.send_thread,
-        )
-        recognition_thread.daemon = True
-        recognition_thread.start()
+        if self.reconnecting:
+            self.reconnecting = False
+            self.ws = self.RecognizerWebSocket(self.__ws_uri, self)
+           
+        if self.ws:
+            self.ws.connect()
+            recognition_thread = threading.Thread(
+                target=self.send_thread,
+            )
+            recognition_thread.daemon = True
+            recognition_thread.start()
 
     def start_continuous_recognition(self):
         """Start continuous speech recognition in synchronized way"""
@@ -361,13 +419,9 @@ class Recognizer(WebSocketClient):
         self.seavoice_stt_loop = asyncio.new_event_loop()
         asyncio.set_event_loop(self.seavoice_stt_loop)
         try:
-            self.seavoice_stt_loop.run_until_complete(
-                self.await_continuous_recognition_async()
-            )
+            self.seavoice_stt_loop.run_until_complete(self.await_continuous_recognition_async())
         except Exception as e:
-            svc_logger.error(
-                f"Exception in thread_start_continuous_recognition_async: {e}"
-            )
+            svc_logger.error(f"Exception in thread_start_continuous_recognition_async: {e}")
         finally:
             self.seavoice_stt_loop.close()
 
@@ -414,32 +468,36 @@ class Recognizer(WebSocketClient):
         except Exception as e:
             svc_logger.error(f"Exception in continuous_recognition_async: {e}")
 
-    def opened(self):
-        """Open websocket"""
-        svc_logger.info("websocket opened")
-        if self.__session_started_event.is_set:
-            evt = {"session_id": self.session_id, "status": 0, "message": "recognition started"}
-            evt_res = EventResults(evt)
-            self.__session_started_event.callback(evt_res)
-
-    def closed(self, code, reason):
-        """Close websocket"""
-        svc_logger.info(f"Closed down: {code}, {reason}")
-        if self.__session_stopped_event.is_set:
-            evt = {"session_id": self.session_id, "status": code, "messge": reason}
-            evt_res = EventResults(evt)
-            self.__session_stopped_event.callback(evt_res)
-        self.close()
-
     def send_thread(self):
         """Send data to websocket"""
         error_flag = False
+
+        if self.monitor_session:
+            data = {
+                "event": "start",
+                "connection_id": self.connection_id,
+                "current_time": self.seconds_received
+            }
+            data = json.dumps(data)
+            self.ws.send(data)
+            svc_logger.info("Send the starting package")
         while self.is_running:
             try:
                 if self.__input_stream is not None:
                     buf = self.__input_stream.read_wait()
-                    if buf is not None and len(buf) > 0:
-                        self.send(buf, binary=True)
+
+                    if not buf or len(buf) == 0:
+                        continue
+
+                    self.update_seconds_received(len(buf))
+                    
+                    if not self.ws.stream:
+                        svc_logger.warning("The ws has been disconnected, trying to reconnect..")
+                        self.reconnecting = True
+                        self.start_continuous_recognition()
+
+                    self.ws.send(buf, binary=True)
+
             except KeyboardInterrupt:
                 error_flag = True
                 self.stop_continuous_recognition_async()
@@ -449,50 +507,30 @@ class Recognizer(WebSocketClient):
                 self.stop_continuous_recognition_async()
         try:
             # inform server that client has finished sending all audio data
-            self.send("EOS")
+            if self.monitor_session:
+                data = {
+                    "event": "end",
+                    "connection_id": self.connection_id,
+                }
+                data = json.dumps(data)
+                self.ws.send(data)
+
+            self.ws.send("EOS")
         except Exception:
             # if can't send 'EOS' to server, actively close websocket connection with server
             error_flag = True
-            self.close()
+            self.ws.close()
         finally:
             if error_flag:
                 if self.__canceled_event.is_set:
-                    evt = {"session_id": self.session_id, "status": ERROR_IN_SEND_THREAD, "messge": "Error while sending data to server"}
+                    evt = {
+                        "session_id": self.session_id,
+                        "status": ERROR_IN_SEND_THREAD,
+                        "messge": "Error while sending data to server",
+                    }
                     evt_res = EventResults(evt)
                     self.__canceled_event.callback(evt_res)
 
-    def received_message(self, message) -> None:
-        """Receive result from websocket"""
-        try:
-            evt = json.loads(str(message))
-            if 'id' in evt:
-                self.session_id = evt['id']
-            if evt['status'] == 0:
-                if 'result' in evt:
-                    if evt['result']['final']:
-                        if self.__recognized_event.is_set:
-                            evt_res = EventResults(evt)
-                            self.__recognized_event.callback(evt_res)
-                            if self.__recognize_once is True:
-                                self.__recognize_once = False
-                                self.close()
-                    else:
-                        if self.__recognizing_event.is_set:
-                            evt_res = EventResults(evt)
-                            self.__recognizing_event.callback(evt_res)
-            else:
-                if 'message' in evt:
-                    evt['message'] = "Received error from server: " + f"{evt['message']}"
-                else:
-                    evt['message'] = "Got error from server."
-                raise RuntimeError(evt)
-        except Exception as e:
-            svc_logger.error(f"Exception in receive_message: {evt}")
-            if self.__canceled_event.is_set:
-                evt_res = EventResults(evt)
-                self.__canceled_event.callback(evt_res)
-            self.stop_continuous_recognition_async()
-
     def stop_continuous_recognition_async(self):
         """Stop recognition"""
         self.is_running = False
@@ -502,6 +540,13 @@ class Recognizer(WebSocketClient):
         """Stop recognition"""
         self.stop_continuous_recognition_async()
         self.__recognizer_semaphore.release()
+    
+    def update_seconds_received(self, buf_len):
+        """ Updates number of audio seconds received"""
+        buf_len_in_seconds = (buf_len
+                              / (self.__input_stream.stream_format.bits_per_sample / 8)
+                              / self.__input_stream.stream_format.samples_per_second)
+        self.seconds_received += buf_len_in_seconds
 
     @property
     def speech_config(self) -> SpeechConfig:
@@ -567,19 +612,11 @@ class Recognizer(WebSocketClient):
 class SpeechRecognizer(Recognizer):
     """Speech recognize"""
 
-    def __init__(
-        self,
-        speech_config: SpeechConfig,
-        audio_config: Optional[audio.AudioConfig] = None
-    ):
+    def __init__(self, speech_config: SpeechConfig, audio_config: Optional[audio.AudioConfig] = None, monitor_session: bool = False, itn: bool = True, punctuation: bool = True):
         """Initialze speech recognizer"""
-        super().__init__(
-            speech_config=speech_config,
-            audio_config=audio_config
-        )
+        super().__init__(speech_config=speech_config, audio_config=audio_config, monitor_session=monitor_session, itn=itn, punctuation=punctuation)
 
-
-class EventBase():
+class EventBase:
     """Base class for events"""
 
     def __init__(self):
@@ -612,7 +649,7 @@ class EventSignal(EventBase):
         super().__init__()
 
 
-class EventResults():
+class EventResults:
     """Simulate MS SessionEventArgs class"""
 
     def __init__(self, evt):
@@ -634,13 +671,13 @@ class EventResults():
             self.result = Results(evt.get("result"))
         if "id" in evt:
             self.session_id = evt.get("id")
-        if 'segment-start' in evt:
+        if "segment-start" in evt:
             self.segment_start = evt.get("segment-start")
-        if 'segment-length' in evt:
+        if "segment-length" in evt:
             self.segment_length = evt.get("segment-length")
-        if 'total-length' in evt:
+        if "total-length" in evt:
             self.total_length = evt.get("total-length")
-        if 'message' in evt:
+        if "message" in evt:
             self.message = evt.get("message")
 
     def __str__(self):
@@ -654,7 +691,7 @@ class EventResults():
         )
 
 
-class Results():
+class Results:
     """Simulate MS SpeechRecognitionEventArgs class"""
 
     def __init__(self, results):
@@ -666,27 +703,22 @@ class Results():
         self.final = None
         self.word_alignment = None
 
-        if 'result_id' in results:
+        if "result_id" in results:
             self.result_id = None
-        if 'hypotheses' in results:
-            self.text = results['hypotheses'][0]['transcript']
-            if 'word-alignment' in results['hypotheses'][0]:
-                self.word_alignment = results['hypotheses'][0]['word-alignment']
-            if 'displaytext' in results['hypotheses'][0]:
-                self.displaytext = results['hypotheses'][0]['displaytext']
-        if 'reason' in results:
+        if "hypotheses" in results:
+            self.text = results["hypotheses"][0]["transcript"]
+            if "word-alignment" in results["hypotheses"][0]:
+                self.word_alignment = results["hypotheses"][0]["word-alignment"]
+            if "displaytext" in results["hypotheses"][0]:
+                self.displaytext = results["hypotheses"][0]["displaytext"]
+        if "reason" in results:
             self.reason = None
-        if 'final' in results:
-            self.final = results['final']
+        if "final" in results:
+            self.final = results["final"]
 
     def __str__(self):
         """Return result info"""
-        return (
-            f"result_id={self.result_id}, "
-            f"text={self.text}, "
-            f"reason={self.reason}, "
-            f"final={self.final}"
-        )
+        return f"result_id={self.result_id}, " f"text={self.text}, " f"reason={self.reason}, " f"final={self.final}"
 
 
 class ResultReason(enum.Enum):
@@ -696,18 +728,12 @@ class ResultReason(enum.Enum):
     ResultReason_SynthesizingAudioCompleted = 12
 
 
-class SpeechSynthesisResult():
+class SpeechSynthesisResult:
     """
     Result of a speech synthesis operation.
     """
 
-    def __init__(
-            self,
-            audio,
-            reason,
-            result_id=None,
-            cancellation_details=None
-    ):
+    def __init__(self, audio, reason, result_id=None, cancellation_details=None):
         """
         Constructor for SpeechSynthesisResult.
         """
@@ -747,14 +773,16 @@ class SpeechSynthesisResult():
         return self._audio_data
 
     def __str__(self):
-        return u'{}(result_id={}, reason={}, audio_length={})'.format(
-            type(self).__name__, self._result_id, self._reason, len(self._audio_data))
+        return "{}(result_id={}, reason={}, audio_length={})".format(
+            type(self).__name__, self._result_id, self._reason, len(self._audio_data)
+        )
 
 
-class ResultFuture():
+class ResultFuture:
     """
     The result of an asynchronous operation.
     """
+
     def __init__(self, client):
         """
         constructor
@@ -774,30 +802,21 @@ class SlothWebsocketParam(object):
         self.business_args = {
             "language": speech_config.speech_synthesis_language,
             "voice": speech_config.speech_synthesis_voice_name,
-            "token": token
+            "token": token,
         }
         self.settings_args = {
             "pitch": speech_config.speech_synthesis_output_pitch,
             "speed": speech_config.speech_synthesis_output_speed,
-            "sample_rate": speech_config.speech_synthesis_output_format['sample_rate'],
+            "sample_rate": speech_config.speech_synthesis_output_format["sample_rate"],
         }
-        self.data = {"text": str(base64.b64encode(text.encode('utf-8')), "UTF8"),
-                     "ssml": is_ssml}
+        self.data = {"text": str(base64.b64encode(text.encode("utf-8")), "UTF8"), "ssml": is_ssml}
 
 
-class SlothWebsocketClient():
-    def __init__(
-            self,
-            speech_config: SpeechConfig,
-            output_stream=None,
-            output_filename=None
-    ):
+class SlothWebsocketClient:
+    def __init__(self, speech_config: SpeechConfig, output_stream=None, output_filename=None):
         websocket.enableTrace(False)
         self.__speech_config = speech_config
-        ws = websocket.WebSocketApp("",
-                                    on_message=self.on_message,
-                                    on_error=self.on_error,
-                                    on_close=self.on_close)
+        ws = websocket.WebSocketApp("", on_message=self.on_message, on_error=self.on_error, on_close=self.on_close)
         self.ws = ws
         self.ws.on_open = self.on_open
         self.__audio_buf = io.BytesIO()
@@ -808,29 +827,20 @@ class SlothWebsocketClient():
         self.__output_filename = output_filename
 
     def save_wav_file(self, audio_pcm, filename):
-        audio = Wave(filename, 'wb')
+        audio = Wave(filename, "wb")
         audio.setnchannels(1)
         audio.setsampwidth(2)
-        audio.setframerate(self.__speech_config.speech_synthesis_output_format['sample_rate'])
+        audio.setframerate(self.__speech_config.speech_synthesis_output_format["sample_rate"])
         audio.writeframes(audio_pcm)
         audio.close()
 
-    def sloth_speak_text(self,
-                         host: str,
-                         token: str,
-                         input_text: str,
-                         is_ssml='False'):
-        self.ws_param = SlothWebsocketParam(speech_config=self.__speech_config,
-                                            token=token,
-                                            text=input_text, is_ssml=is_ssml)
-        self.ws.url=host
+    def sloth_speak_text(self, host: str, token: str, input_text: str, is_ssml="False"):
+        self.ws_param = SlothWebsocketParam(speech_config=self.__speech_config, token=token, text=input_text, is_ssml=is_ssml)
+        self.ws.url = host
         self.ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
         self.__audio_buf.seek(0)
         audio_data = self.__audio_buf.read()
-        self.speech_synthesis_result = SpeechSynthesisResult(
-            audio_data,
-            ResultReason.ResultReason_SynthesizingAudioCompleted
-        )
+        self.speech_synthesis_result = SpeechSynthesisResult(audio_data, ResultReason.ResultReason_SynthesizingAudioCompleted)
         if self.__output_stream is not None:
             self.__output_stream.write(audio_data)
         if self.__output_filename is not None:
@@ -838,15 +848,8 @@ class SlothWebsocketClient():
         self.semaphore.release()
         return self.speech_synthesis_result
 
-    def sloth_speak_text_async(self,
-                               host: str,
-                               token: str,
-                               input_text: str,
-                               is_ssml='False'):
-        task_speak = threading.Thread(
-            target=self.sloth_speak_text,
-            args=(host, token, input_text, is_ssml)
-        )
+    def sloth_speak_text_async(self, host: str, token: str, input_text: str, is_ssml="False"):
+        task_speak = threading.Thread(target=self.sloth_speak_text, args=(host, token, input_text, is_ssml))
         task_speak.daemon = True
         task_speak.start()
         return ResultFuture(self)
@@ -857,7 +860,7 @@ class SlothWebsocketClient():
             status = message["status"]
             progress = message["progress"]
             sid = message["sid"]
-            if status is not "ok":
+            if status != "ok":
                 svc_logger.info(f"Sloth websocket: sid: {sid}; status: {status}; progress:{progress}.")
 
             audio = message["data"]["audio"]
@@ -866,7 +869,8 @@ class SlothWebsocketClient():
             if ending_status == 2:
                 self.__audio_buf.write(audio)
                 self.ws.close()
-                #self.__running = False # comment out on 2021/7/27, to support calling speak_text multiple times, with this, sdk will only call synthesizing callback for the first text
+                # self.__running = False # comment out on 2021/7/27, to support calling speak_text multiple times, with this,
+                # sdk will only call synthesizing callback for the first text
 
                 if self.synthesis_completed is not None:
                     if self.synthesis_completed.is_set is True:
@@ -895,9 +899,10 @@ class SlothWebsocketClient():
 
     def on_open(self):
         def run(*args):
-            data = {"business": self.ws_param.business_args,
-                    "settings": self.ws_param.settings_args,
-                    "data": self.ws_param.data,
+            data = {
+                "business": self.ws_param.business_args,
+                "settings": self.ws_param.settings_args,
+                "data": self.ws_param.data,
             }
             data = json.dumps(data)
             svc_logger.info(f"Sloth websocket: start sending data: {data}")
@@ -921,21 +926,20 @@ class SpeechSynthesizer(SlothWebsocketClient):
         If it is None, the output audio will be dropped.
         None can be used for scenarios like performance test.
     """
-    def __init__(
-            self,
-            speech_config: SpeechConfig,
-            audio_config: Optional[audio.AudioOutputConfig] = None
-    ):
+
+    def __init__(self, speech_config: SpeechConfig, audio_config: Optional[audio.AudioOutputConfig] = None):
         """Initialize speech recognizer"""
         if not isinstance(speech_config, SpeechConfig):
-            raise ValueError('speech_config must be a SpeechConfig instance')
+            raise ValueError("speech_config must be a SpeechConfig instance")
 
         # load speech_config
         self.__speech_config = speech_config
         self.__format_label = speech_config.format_label
         self.__account_id = speech_config.account_id
         if speech_config.get_login_token() is False:
-            svc_logger.error(f"Failed to login API server for {self.__account_id}, please make sure account_id and password are right.")
+            svc_logger.error(
+                f"Failed to login API server for {self.__account_id}, please make sure account_id and password are right."
+            )
             return None
         self.__login_token = speech_config.login_token
         self.__speech_synthesis_language = speech_config.speech_synthesis_language
@@ -965,16 +969,21 @@ class SpeechSynthesizer(SlothWebsocketClient):
 
     def get_server_url_token(self) -> bool:
         """get tts server url and speech token by login token"""
-        headers = {'Content-Type': 'text/plain', 'token': self.__login_token}
-        response = requests.post(API_SERVER_TTS_SERVICE_URL, headers=headers,
-                                 json={'language': self.__speech_synthesis_language, 'voice': self.__speech_synthesis_voice_name})
+        headers = {"Content-Type": "text/plain", "token": self.__login_token}
+        response = requests.post(
+            API_SERVER_TTS_SERVICE_URL,
+            headers=headers,
+            json={"language": self.__speech_synthesis_language, "voice": self.__speech_synthesis_voice_name},
+        )
         if response.status_code == 200:
             response_json = response.json()
-            self.__host = response_json['server_url']
-            self.__token = response_json['token']
+            self.__host = response_json["server_url"]
+            self.__token = response_json["token"]
             return True
         else:
-            svc_logger.error(f"Error while getting tts server and token: {response.status_code} for account_id: {self.__account_id}")
+            svc_logger.error(
+                f"Error while getting tts server and token: {response.status_code} for account_id: {self.__account_id}"
+            )
             return False
 
     def speak_text(self, text: str) -> Optional[SpeechSynthesisResult]:
@@ -984,9 +993,9 @@ class SpeechSynthesizer(SlothWebsocketClient):
         :return: A SpeechSynthesisResult.
         """
         if self.get_server_url_token() is True:
-            return super().sloth_speak_text(self.__host, self.__token, text, is_ssml='False')
+            return super().sloth_speak_text(self.__host, self.__token, text, is_ssml="False")
         else:
-            svc_logger.error(f"Can't get an available tts server, please confirm language and voice settings, then try again.")
+            svc_logger.error("Can't get an available tts server, please confirm language and voice settings, then try again.")
             return None
 
     def speak_ssml(self, ssml: str) -> Optional[SpeechSynthesisResult]:
@@ -996,9 +1005,9 @@ class SpeechSynthesizer(SlothWebsocketClient):
         :return: A SpeechSynthesisResult.
         """
         if self.get_server_url_token() is True:
-            return super().sloth_speak_text(self.__host, self.__token, ssml, is_ssml='True')
+            return super().sloth_speak_text(self.__host, self.__token, ssml, is_ssml="True")
         else:
-            svc_logger.error(f"Can't get an available tts server, please confirm language and voice settings, then try again.")
+            svc_logger.error("Can't get an available tts server, please confirm language and voice settings, then try again.")
             return None
 
     def speak_text_async(self, text: str) -> Optional[ResultFuture]:
@@ -1008,9 +1017,9 @@ class SpeechSynthesizer(SlothWebsocketClient):
         :return: A future with SpeechSynthesisResult.
         """
         if self.get_server_url_token() is True:
-            return super().sloth_speak_text_async(self.__host, self.__token, text, is_ssml='False')
+            return super().sloth_speak_text_async(self.__host, self.__token, text, is_ssml="False")
         else:
-            svc_logger.error(f"Can't get an available tts server, please confirm language and voice settings, then try again.")
+            svc_logger.error("Can't get an available tts server, please confirm language and voice settings, then try again.")
             return None
 
     def speak_ssml_async(self, ssml: str) -> Optional[ResultFuture]:
@@ -1020,9 +1029,9 @@ class SpeechSynthesizer(SlothWebsocketClient):
         :return: A future with SpeechSynthesisResult.
         """
         if self.get_server_url_token() is True:
-            return super().sloth_speak_text_async(self.__host, self.__token, ssml, is_ssml='True')
+            return super().sloth_speak_text_async(self.__host, self.__token, ssml, is_ssml="True")
         else:
-            svc_logger.error(f"Can't get an available tts server, please confirm language and voice settings, then try again.")
+            svc_logger.error("Can't get an available tts server, please confirm language and voice settings, then try again.")
             return None
 
     def start_speaking_text(self, text: str) -> SpeechSynthesisResult:
